{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-object-modal{% endblock %}
{% block modal_title %}Delete {{ data.object_name }}{% endblock %}

{% block modal_body %}
{% endblock %}

{% block modal_footer %}
<a class="btn btn-default pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i>
    Cancel
</a>
<a id="delete-object-save" class="btn btn-danger" href="#"><i class="fa fa-trash"></i> Delete</a>
{% endblock %}